/* BeaSTM (c) Thorbjörn Jemander */

#ifndef STM_RANGE_SLIDER_H
#define STM_RANGE_SLIDER_H

#define STM_ARROW_UP		0
#define STM_ARROW_RIGHT		1
#define STM_ARROW_DOWN		2
#define STM_ARROW_LEFT		3

class JRangeSlider;

#include <ScrollBar.h>
#include <View.h>
#include <interface/GraphicsDefs.h>

#define min(x,y) (x>y?y:x)
#define max(x,y) (x>y?x:y)
#define ONE_UNIT BPoint(1,1)

class JRangeSlider : public BView
{
	virtual
	void 		Draw( BRect argUpdateRect );
	virtual
	void		MouseDown( BPoint argDownPoint );

	void		CalculateNewRectangles();
	float		HeightOfSlider();
	bool		BoundUpperAndLowerPositions( 	float argUpperPosition, float argLowerPosition );

	void		IncUppAndLowWhileLMBPressed(	uint32	argButtons, BPoint argDownPoint, float argStep, bool argRepetitive = true);		// LMB = Left Mouse Button (Primary Mouse Button)
	void		DecUppAndLowWhileLMBPressed(	uint32	argButtons, BPoint argDownPoint, float argStep, bool argRepetitive = true);		// RMB	= Right Mouse Button (Secondary Mouse Button)
	void		IncUppWhileRMBPressed(			uint32	argButtons, BPoint argDownPoint, float argStep, bool argRepetitive = true);		// AMB	= Any Mouse Button
	void		DecUppWhileRMBPressed(			uint32	argButtons, BPoint argDownPoint, float argStep, bool argRepetitive = true);
	void		IncLowWhileRMBPressed(			uint32	argButtons, BPoint argDownPoint, float argStep, bool argRepetitive = true);
	void		DecLowWhileRMBPressed(			uint32	argButtons, BPoint argDownPoint, float argStep, bool argRepetitive = true);
	void		UppAndLowFollowMouseWhileAMBPressed(uint32	argButtons, BPoint argDownPoint);
	void		UppFollowMouseWhileAMBPressed(	uint32	argButtons, BPoint argDownPoint);
	void		LowFollowMouseWhileAMBPressed(	uint32	argButtons, BPoint argDownPoint);

	float		UpperPosition,	// Relative value 0-1.
				LowerPosition,	// Relative value 0-1.
				Width,			// WIdth of view nn pixels.
				Height,			// Height of view in pixels.
				DisplayRange,	// The height of the container in pixels.
				SmallStep,		// Step the slider takes when pressing an arrow button.
				LargeStep,		// Larger step. Will maybe implemented.
				KnobSize;		// In pixels.
	
	orientation	Direction;

	BRect		ArrowUpAboveSlider,
				ArrowDownAboveSlider,
				ContainerAboveSlider,
				SliderTopSizeKnob,
				SliderMoveKnob,
				SliderBottomSizeKnob,
				ContainerBelowSlider,
				ArrowUpBelowSlider,
				ArrowDownBelowSlider;

	bool		ArrowUpAboveSliderPressed,
				ArrowDownAboveSliderPressed,
				ContainerAboveSliderPressed,
				SliderTopSizeKnobPressed,
				SliderMoveKnobPressed,
				SliderBottomSizeKnobPressed,
				ContainerBelowSliderPressed,
				ArrowUpBelowSliderPressed,
				ArrowDownBelowSliderPressed;

public:

	rgb_color	DarkGrey,
				MediumDarkGrey,	
				MediumLightGrey,
				LightGrey,
				White;

	bool	SetUpperPosition( float argValue );	
	bool	SetLowerPosition( float argValue );	
	void	SetSmallStep( float argValue )		{ SmallStep = argValue; }
	void	SetLargeStep( float argValue )		{ LargeStep = argValue; }

			JRangeSlider(	BRect		argFrame,
							const char	*argName,
							float		argUpperPosition,
							float		argLowerPosition,
							orientation	argDirection,
							float		argSmallStep = 0.005);

	virtual
	void		DrawInRect(	BRect argRect, bool	argActivated);
	virtual
	void		DrawOutRect(BRect argRect, bool argActivated);
	virtual
	void		DrawArrowIn(BRect argRect, int32 argDirection, bool argActivated); 
	virtual
	void		UpperPositionChanged(	float argNewUpperPosition );
	virtual	
	void		LowerPositionChanged( 	float argNewLowerPosition );
	virtual
	void		BothUpperAndLowerPositionsChanged( 	float argNewUpperPosition, 	float argNewLowerPosition  );
};

#endif